--- lts - lua thread analogue
--author: fokichevskiy
--blasthack: https://www.blast.hk/members/533688/

local lts = {}

local threads = {}

function wait(time)
	coroutine.yield(time / 1000)
end

function lts.create(func, ...)
	local aaa = #threads+1
	threads[aaa] = {
		func = coroutine.create(func),
		arguments = {...},
		awake = os.clock(),
		paused = false,
		terminate = function()
			threads[aaa] = nil
		end,
		status = function(self)
			return coroutine.status(self.func)
		end,
		pause = function(self)
			self.paused = true
		end,
		run = function(self)
			self.paused = false
		end,
		resum = function(self)
			if self.paused or self.awake > os.clock() then
				return
			end
			if self:status() == 'dead' then
				self:terminate()
				return
			end
			local resumed, result = coroutine.resume(self.func, table.unpack(self.arguments))
			if not resumed then
				error(result, 2)
			elseif result then
				self.awake = os.clock() + result
			end
		end
	}
	return threads[aaa]
end

function lts.create_suspended(func, ...)
	local aaa = #threads+1
	threads[aaa] = {
		func = coroutine.create(func),
		arguments = {...},
		awake = os.clock(),
		paused = true,
		terminate = function()
			threads[aaa] = nil
		end,
		status = function(self)
			return coroutine.status(self.func)
		end,
		pause = function(self)
			self.paused = true
		end,
		run = function(self)
			self.paused = false
		end,
		resum = function(self)
			if self.paused or self.awake > os.clock() then
				return
			end
			if self:status() == 'dead' then
				self:terminate()
				return
			end
			local resumed, result = coroutine.resume(self.func, table.unpack(self.arguments))
			if not resumed then
				error(result, 2)
			elseif result then
				self.awake = os.clock() + result
			end
		end
	}
	return threads[aaa]
end

function lts.killAll()
	threads = {}
end

addEventHandler('onUpdate', function() -- addEventHandler       function onUpdate()   function onUpdate(),     
	for k, v in pairs(threads) do
		v:resum()
	end
end)

return lts
